# encoding=utf-8
# Time: 5/2/17
# File: __init__.py.py
# Author: jian<jian@mltalker.com>
from __future__ import division
from __future__ import unicode_literals
from __future__ import print_function

from .simplesyn import simple_syn_sample

__all__ = ['simple_syn_sample']